<?php

class HexParser
{
  var $_pos;
  var $_str;
  var $_err;

  function HexParser($InitStr)
  {
    $this->_str = $InitStr;
    $this->_pos = 1;
    $this->_err = -1;
  }

  function GetFlags()
  {
    return ord($this->_str[0]);
  }

  function GetByte()
  {
    if (strlen($this->_str) > $this->_pos+1) 
    {
      $str = substr($this->_str, $this->_pos, 2);
      $this->_pos += 2;
      if (ctype_xdigit($str))
        return intval($str, 16);
    }
    if ($this->_err < 0) $this->_err = $this->_pos;
    return -1;
  }

  function GetValue($bL)
  {
    $val = $this->GetByte();
    if ($bL)
    {
      $val = ($val << 8) | $this->GetByte();
    }
    return $val;
  }
}

function CreateSignatureImage($sign, $nSizeX, $nSizeY)
{

  $parser = new HexParser($sign);

  $bFlags = $parser->GetFlags();
  if ($bFlags < 0) return "ERROR";
  $bLong = ($bFlags & 0x40) != 0;

  $nCurves = $parser->GetValue(true);
  $nPoints = $parser->GetValue(true);
  $nWidth = $parser->GetValue(true);
  $nHeight = $parser->GetValue(true);

  $nOfsX = 0;
  $nOfsY = 0;
  $fScale = 1.0;

  if ($nSizeX > 0)
  {
    // Width is determined
    if ($nSizeY > 0)
    {
      // Both new width and height are specified
      // Try to inscribe signature curves into specified rectangle
      $fOrgRatio = floatval($nWidth)/floatval($nHeight);
      if ($fOrgRatio < floatval($nSizeX)/floatval($nSizeY))
      {
        $fScale = floatval($nSizeY)/floatval($nHeight);
        $nOfsX = round((floatval($nSizeX)-$fScale*floatval($nWidth))/2.0);
      }
      else
      {
        $fScale = floatval($nSizeX)/floatval($nWidth);
        $nOfsY = round((floatval($nSizeY)-$fScale*floatval($nHeight))/2.0);
      }
      $nWidth = $nSizeX;
      $nHeight = $nSizeY;
    }
    else
    {
      $fScale = floatval($nSizeX)/floatval($nWidth);
      $nWidth = $nSizeX;
      $nHeight = round(floatval($nHeight)*$fScale);
    }
  }
  else
  {
    // Width is not specified
    if ($nSizeY > 0)
    {
      $fScale = floatval($nSizeY)/floatval($nHeight);
      $nHeight = $nSizeY;
      $nWidth = round(floatval($nWidth)*$fScale);
    }
  }


  $img = imageCreate($nWidth, $nHeight);

  $black = imageColorAllocate($img, 0, 0, 0);
  $white = imageColorAllocate($img, 255, 255, 255);
  $red = imageColorAllocate($img, 255, 0, 0);

  if (isset($nWidth) && isset($nHeight))
  {
    for ($i = 0; ($i < $nCurves) && ($parser->_err < 0); $i++)
    {
      $nCurPts = $parser->GetValue(true);
      if ($nCurPts > 0)
      {
        for ($k = 0; ($k < $nCurPts) && ($parser->_err < 0); $k++)
        {
          $pts[$i][] = $parser->GetValue($bLong);
          $pts[$i][] = $parser->GetValue($bLong);
        }
        $nPoints -= $nCurPts;
      }
    }

    imageFilledRectangle($img, 0, 0, $nWidth, $nHeight, $white);

    for ($i = 0; $i < $nCurves; $i++)
    {
      if (isset($pts[$i]))
      {
        $points = $pts[$i];
        $npts = (int) count($points)/2;

        $x1 = $nOfsX + round(intval(floatval($points[0])*$fScale));
        $y1 = $nOfsY + round(intval(floatval($points[1])*$fScale));
        for ($k = 1; $k < $npts; $k++)
        {
          $x2 = $nOfsX + round(floatval($points[$k*2-0])*$fScale);
          $y2 = $nOfsY + round(floatval($points[$k*2+1])*$fScale);
          imageLine($img, $x1, $y1, $x2, $y2, $black);
          $x1 = $x2;
          $y1 = $y2;
        }
      }
    }
  }

  if ($parser->_err > 0)
  {
    imageString ($img, 3, 2, 2, "ERROR:" . $parser->_err, $red);
  }
  return $img;
}

?>
